package com.xydl.cac.task;

import com.xydl.cac.entity.*;
import com.xydl.cac.iec.IEDService;
import com.xydl.cac.repository.*;
import com.xydl.cac.service.DataService;
import com.xydl.cac.service.ModevTypeService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

@Service
@Slf4j
public class Client61850Task {

    @Resource
    IcdFileRepository fileRepository;
    @Resource
    IcdIedRepository iedRepository;
    @Resource
    IcdConfigTypeRepository configRepository;
    @Resource
    IcdConfigTypeAttRepository attRepository;
    @Resource
    IcdConfigTypeInstRepository instRepository;
    @Resource
    RptparamindexRepository rptparamindexRepository;
    @Resource
    DataService dataService;
    @Resource
    ModevTypeService modevTypeService;
    @Resource
    NSensorRepository sensorRepository;

    @Scheduled(cron = "0 * * * * ?")
    public void collectAll() {
        Calendar cal = Calendar.getInstance();
        int h = cal.get(Calendar.HOUR_OF_DAY);
        int m = cal.get(Calendar.MINUTE);
        m = h * 60 + m;

        List<Integer> typeIdList = new ArrayList<>();
        List<ModevType> modevTypeList = modevTypeService.listAll();
        for (ModevType type : modevTypeList) {
            int intervals = 30;
            if (type.getIntervals() != null) {
                intervals = type.getIntervals();
            }
            int x = m % intervals;
            if (x == 0) {
                typeIdList.add(type.getId());
            }
        }
        if (CollectionUtils.isEmpty(typeIdList)) {
            return;
        }

        List<NSensor> sensorList = sensorRepository.findByTypeIdIn(typeIdList);
        List<Integer> eqmidList = new ArrayList<>();
        for (NSensor sensor : sensorList) {
            if (sensor.getDevId() != null) {
                eqmidList.add(sensor.getDevId());
            }
        }
        if (CollectionUtils.isEmpty(eqmidList)) {
            return;
        }

        List<Rptparamindex> rptList = rptparamindexRepository.findAllByEqmidInAndColnameIsNotNull(eqmidList);
        if (CollectionUtils.isEmpty(rptList)) {
            return;
        }

        List<IcdFile> icdFileList = fileRepository.findAll();
        if (!CollectionUtils.isEmpty(icdFileList)) {
            for (IcdFile icdFile : icdFileList) {
                this.collectIcdFile(icdFile, rptList);
            }
        }
    }

    private void collectIcdFile(IcdFile icdFile, List<Rptparamindex> rptList) {
        List<IcdIed> iedList = iedRepository.findByIcdFileId(icdFile.getId());
        if (!CollectionUtils.isEmpty(iedList)) {
            for (IcdIed ied : iedList) {
                this.collectIed(icdFile.getXml(), ied, rptList);
            }
        }
    }

    private void collectIed(String xml, IcdIed ied, List<Rptparamindex> rptList) {
        IEDService iedService = new IEDService(configRepository, attRepository, instRepository,
                rptparamindexRepository, dataService, xml, ied);

        iedService.collectAndSave(rptList);
    }

}
